/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2010-2011  Oleg Linkin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/

#include "acetamide.h"
#include <ctime>
#include <QIcon>
#include <QStandardItemModel>
#include <xmlsettingsdialog/xmlsettingsdialog.h>
#include <util/util.h>
#include "core.h"
#include "xmlsettingsmanager.h"
#include "nickservidentifywidget.h"

namespace LeechCraft
{
namespace Azoth
{
namespace Acetamide
{
	void Plugin::Init (ICoreProxy_ptr proxy)
	{
		Translator_.reset (Util::InstallTranslator ("azoth_acetamide"));

		qsrand (time (NULL));

		qRegisterMetaTypeStreamOperators<QList<QStringList>> ("QList<QStringList>");

		SettingsDialog_.reset (new Util::XmlSettingsDialog);
		SettingsDialog_->RegisterObject (&XmlSettingsManager::Instance (),
					"azothacetamidesettings.xml");

		Core::Instance ().Init ();
		Core::Instance ().SetProxy (proxy);

		SettingsDialog_->SetCustomWidget ("NickServIdentifyWidget",
				Core::Instance ().GetNickServIdentifyWidget ());

		connect (&Core::Instance (),
				SIGNAL (gotEntity (const LeechCraft::Entity&)),
				this,
				SIGNAL (gotEntity (const LeechCraft::Entity&)));
	}

	void Plugin::SecondInit ()
	{
		Core::Instance ().SecondInit ();
	}

	void Plugin::Release ()
	{
		Core::Instance ().Release ();
	}

	QByteArray Plugin::GetUniqueID () const
	{
		return "org.LeechCraft.Azoth.Acetamide";
	}

	QString Plugin::GetName () const
	{
		return "Azoth Acetamide";
	}

	QString Plugin::GetInfo () const
	{
		return tr ("IRC protocol support.");
	}

	QIcon Plugin::GetIcon () const
	{
		static QIcon icon (":/plugins/azoth/plugins/acetamide/resources/images/acetamide.svg");
		return icon;
	}

	QSet<QByteArray> Plugin::GetPluginClasses () const
	{
		QSet<QByteArray> classes;
		classes << "org.LeechCraft.Plugins.Azoth.Plugins.IProtocolPlugin";
		return classes;
	}

	QObject* Plugin::GetObject ()
	{
		return this;
	}

	QList<QObject*> Plugin::GetProtocols () const
	{
		return Core::Instance ().GetProtocols ();
	}

	Util::XmlSettingsDialog_ptr Plugin::GetSettingsDialog () const
	{
		return SettingsDialog_;
	}

	void Plugin::initPlugin (QObject *proxy)
	{
		Core::Instance ().SetPluginProxy (proxy);
	}
}
}
}

QDataStream& operator<< (QDataStream& out, const QList<QStringList>& list)
{
	Q_FOREACH (const QStringList& subList, list)
		out << subList;

	return out;
}

QDataStream& operator>> (QDataStream& in, QList<QStringList>& list)
{
	QStringList subList;
	while (!in.atEnd ())
	{
		in >> subList;
		list << subList;
		subList.clear ();
	}

	return in;
}

LC_EXPORT_PLUGIN (leechcraft_azoth_acetamide,
		LeechCraft::Azoth::Acetamide::Plugin);
