/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2012  Georg Rudoy
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/

#ifndef PLUGINS_AZOTH_PLUGINS_HERBICIDE_HERBICIDE_H
#define PLUGINS_AZOTH_PLUGINS_HERBICIDE_HERBICIDE_H
#include <QObject>
#include <interfaces/iinfo.h>
#include <interfaces/iplugin2.h>
#include <interfaces/ihavesettings.h>
#include <interfaces/core/ihookproxy.h>

class QTranslator;

namespace LeechCraft
{
namespace Azoth
{
namespace Herbicide
{
	class ConfWidget;

	class Plugin : public QObject
				 , public IInfo
				 , public IPlugin2
				 , public IHaveSettings
	{
		Q_OBJECT
		Q_INTERFACES (IInfo IPlugin2 IHaveSettings)

		Util::XmlSettingsDialog_ptr SettingsDialog_;
		ConfWidget *ConfWidget_;
		QSet<QObject*> AskedEntries_;
		QSet<QObject*> AllowedEntries_;
		QSet<QObject*> OurMessages_;

		QHash<QObject*, QString> DeniedAuth_;

		QSet<QRegExp> Whitelist_;
		QSet<QRegExp> Blacklist_;
	public:
		void Init (ICoreProxy_ptr);
		void SecondInit ();
		QByteArray GetUniqueID () const;
		void Release ();
		QString GetName () const;
		QString GetInfo () const;
		QIcon GetIcon () const;

		QSet<QByteArray> GetPluginClasses () const;

		Util::XmlSettingsDialog_ptr GetSettingsDialog () const;
	private:
		bool IsConfValid () const;
		bool IsEntryAllowed (QObject*) const;

		void ChallengeEntry (IHookProxy_ptr, QObject*);
		void GreetEntry (QObject*);
	public slots:
		void hookGotAuthRequest (LeechCraft::IHookProxy_ptr proxy,
				QObject *entry,
				QString msg);
		void hookGotMessage (LeechCraft::IHookProxy_ptr proxy,
				QObject *message);
	private slots:
		void handleWhitelistChanged ();
		void handleBlacklistChanged ();
	};
}
}
}

#endif
